@@ -32,6 +32,8 @@
 #include <sys/types.h>
 #include <signal.h>
 
+#include <fontconfig/fontconfig.h>
+
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
@@ -46,6 +48,8 @@
 #include "gdm-socket-protocol.h"
 #include "gdm-daemon-config-keys.h"
 
+#include "pixmaps/24x24/inlines.h"
+
 gint gdm_timed_delay = 0;
 static Atom AT_SPI_IOR;
 
@@ -203,6 +207,49 @@ gdm_common_setup_cursor (GdkCursorType type)
 	gdk_cursor_unref (cursor);
 }
 
+void
+gdm_common_setup_builtin_icons (void)
+{
+        typedef struct 
+        {
+                const char *id;
+                gint size;
+                const guint8 *data;
+        } GdmThemeIcon;
+
+	static const GdmThemeIcon builtins[] =
+	{
+		{"preferences-desktop-locale", 24, preferences_desktop_locale_24},
+		{"preferences-desktop-remote-desktop", 24, preferences_desktop_remote_desktop_24},
+		{"system-log-out", 24, system_log_out_24},
+		{"system-restart", 24, system_restart_24},
+		{"system-shut-down", 24, system_shut_down_24},
+		{"system-suspend", 24, system_suspend_24},
+		{"user-desktop", 24, user_desktop_24}
+	};
+
+	static gboolean icons_setup = FALSE;
+	int i;
+
+	if (icons_setup)
+		return;
+	icons_setup = TRUE;
+
+	for (i = 0; i < (int) G_N_ELEMENTS (builtins); ++i)
+	{
+		GdkPixbuf *pixbuf;
+
+		pixbuf = gdk_pixbuf_new_from_inline (-1, builtins[i].data,
+						     FALSE, NULL);
+
+		gtk_icon_theme_add_builtin_icon (builtins[i].id,
+						 builtins[i].size,
+						 pixbuf);
+
+		g_object_unref (G_OBJECT (pixbuf));
+	}
+}
+
 void
 gdm_common_login_sound (const gchar *GdmSoundProgram,
 			const gchar *GdmSoundOnLoginReadyFile,
@@ -397,7 +444,7 @@ gdm_common_text_to_escaped_utf8 (const char *text)
 	while ((*p != '\0') && 
 	       !g_utf8_validate (p, -1, &q)) {
 		p = (gchar *) q;
-		p = '?';
+		*p = '?';
 		p++;
 	}
  
@@ -928,3 +975,96 @@ gdm_common_expand_text (const gchar *text)
 	return g_string_free (str, FALSE);
 }
 
+typedef enum
+{
+  LOCALE_UP_TO_LANGUAGE = 0,
+  LOCALE_UP_TO_COUNTRY,
+  LOCALE_UP_TO_ENCODING,
+  LOCALE_UP_TO_MODIFIER,
+} LocaleScope;
+
+static char *
+get_less_specific_locale (const char *locale,
+                          LocaleScope scope)
+{
+  char *generalized_locale;
+  char *end;
+
+  generalized_locale = strdup (locale);
+
+  end = strchr (generalized_locale, '_');
+
+  if (end != NULL && scope <= LOCALE_UP_TO_LANGUAGE)
+    {
+      *end = '\0';
+      return generalized_locale;
+    }
+
+  end = strchr (generalized_locale, '.');
+
+  if (end != NULL && scope <= LOCALE_UP_TO_COUNTRY)
+    {
+      *end = '\0';
+      return generalized_locale;
+    }
+
+  end = strchr (generalized_locale, '@');
+
+  if (end != NULL && scope <= LOCALE_UP_TO_ENCODING)
+    {
+      *end = '\0';
+      return generalized_locale;
+    }
+
+  return generalized_locale;
+}
+
+gboolean
+gdm_common_locale_is_displayable (const gchar *locale)
+{
+  char *language_code;
+  gboolean is_displayable;
+
+  FcPattern *pattern;
+  FcObjectSet *object_set;
+  FcFontSet *font_set;
+
+  is_displayable = FALSE;
+  pattern = NULL;
+  object_set = NULL;
+  font_set = NULL;
+
+  language_code = get_less_specific_locale (locale, LOCALE_UP_TO_LANGUAGE);
+
+  pattern = FcPatternBuild (NULL, FC_LANG, FcTypeString, language_code, NULL);
+
+  if (pattern == NULL)
+    goto done;
+
+  object_set = FcObjectSetBuild (NULL, NULL);
+
+  if (object_set == NULL)
+    goto done;
+
+  font_set = FcFontList (NULL, pattern, object_set);
+
+  if (font_set == NULL)
+    goto done;
+
+  is_displayable = (font_set->nfont > 0);
+
+done:
+
+  if (font_set != NULL)
+    FcFontSetDestroy (font_set);
+
+  if (object_set != NULL)
+    FcObjectSetDestroy (object_set);
+
+  if (pattern != NULL)
+    FcPatternDestroy (pattern);
+
+  g_free (language_code);
+  return is_displayable;
+}
+
