@@ -18,19 +18,27 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+#include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "gdm.h"
 #include "gdmconfig.h"
 #include "gdmsession.h"
+#include "gdmlanguages.h"
+
+#include "gdm-common.h"
+#include "gdm-daemon-config-keys.h"
+#include "gdm-socket-protocol.h"
 
 #include "greeter_item.h"
 #include "greeter_configuration.h"
 #include "greeter_item_customlist.h"
 #include "greeter_parser.h"
 #include "greeter_session.h"
-#include "greeter_action_language.h"
+
+#define LAST_LANGUAGE "Last"
 
 /*
  * Keep track of the session/language widgets so we can
@@ -158,9 +166,9 @@ greeter_custom_set_session (gchar *session)
  * the language dialog (launched from language button or F10 menu).
  */
 void
-greeter_custom_set_language (gchar *language)
+lang_set_custom_callback (gchar *language)
 {
-  GtkListStore *lang_model = greeter_language_get_model ();
+  GtkListStore *lang_model = gdm_lang_get_model ();
   GtkTreeIter iter;
   gboolean valid;
   char *locale_name;
@@ -174,7 +182,7 @@ greeter_custom_set_language (gchar *language)
    * Handle for either combo box or list style, depending on which is being
    * used.
  . */
-  if (GTK_IS_COMBO_BOX (session_widget))
+  if (GTK_IS_COMBO_BOX (language_widget))
     {
       valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (lang_model),
          &iter);
@@ -310,9 +318,9 @@ populate_session (GObject * object)
 static void
 populate_language (GObject *object)
 {
-  GtkListStore *lang_model = greeter_language_get_model ();
+  GtkListStore *lang_model = gdm_lang_get_model ();
   GtkTreeIter iter;
-  char *name, *untranslated, *display_name, *locale_name;
+  char *name, *untranslated, *lang_display_name, *locale_name;
   gboolean valid;
 
   valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (lang_model),
@@ -325,12 +333,12 @@ populate_language (GObject *object)
          LOCALE_COLUMN, &locale_name, -1);
 
       if (untranslated)
-         display_name = g_strdup_printf ("%s (%s)", name, untranslated);
+         lang_display_name = g_strdup_printf ("%s (%s)", name, untranslated);
       else
-         display_name = g_strdup (name);
+         lang_display_name = g_strdup (name);
 
       if (GTK_IS_COMBO_BOX (object))
-         gtk_combo_box_append_text (GTK_COMBO_BOX (object), display_name);
+         gtk_combo_box_append_text (GTK_COMBO_BOX (object), lang_display_name);
       else if (GTK_IS_TREE_MODEL (object))
         {
           GtkTreeIter loopiter;
@@ -338,7 +346,7 @@ populate_language (GObject *object)
 
           gtk_list_store_append (GTK_LIST_STORE (tm), &loopiter);
           gtk_list_store_set (GTK_LIST_STORE (tm), &loopiter,
-             GREETER_LIST_TEXT, display_name,
+             GREETER_LIST_TEXT, lang_display_name,
              GREETER_LIST_ID,   locale_name,
              -1);
         }
@@ -347,7 +355,7 @@ populate_language (GObject *object)
 }
 
 /* Callback helper function to set session value */
-void
+static void
 combo_session_selected (char *session_val)
 {
   GList *tmp;
@@ -407,9 +415,9 @@ combo_selected (GtkComboBox *combo, GreeterItemInfo *item)
        * Since combo boxes can't store the ID value, have to do some
        * extra work to figure out which row is selected.
        */
-      GtkListStore *lang_model = greeter_language_get_model ();
+      GtkListStore *lang_model = gdm_lang_get_model ();
       GtkTreeIter iter;
-      char *name, *untranslated, *display_name, *locale_name;
+      char *name, *untranslated, *lang_display_name, *locale_name;
       gboolean valid;
 
       valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (lang_model),
@@ -422,16 +430,16 @@ combo_selected (GtkComboBox *combo, GreeterItemInfo *item)
              LOCALE_COLUMN, &locale_name, -1);
 
           if (untranslated)
-             display_name = g_strdup_printf ("%s (%s)", name, untranslated);
+             lang_display_name = g_strdup_printf ("%s (%s)", name, untranslated);
           else
-             display_name = g_strdup (name);
+             lang_display_name = g_strdup (name);
 
-          if (strcmp (display_name, active) == 0)
+          if (strcmp (lang_display_name, active) == 0)
             {
-              greeter_language_set (locale_name);
+              gdm_lang_set_restart_dialog (locale_name);
               break;
             }
-          g_free (display_name);
+          g_free (lang_display_name);
           valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (lang_model), &iter);
         }
     }
@@ -518,7 +526,7 @@ list_selected (GtkTreeSelection *selection, GreeterItemInfo *item)
   else if (strcmp (item->id, "language") == 0)
     {
       if (id != NULL)
-        greeter_language_set (id);
+        gdm_lang_set_restart_dialog (id);
     }
   else
     {
